/*******************************************************************************

INTEL CORPORATION PROPRIETARY INFORMATION
This software is supplied under the terms of a license agreement or nondisclosure
agreement with Intel Corporation and may not be copied or disclosed except in
accordance with the terms of that agreement
Copyright(c) 2012 Intel Corporation. All Rights Reserved.

*******************************************************************************/
#ifndef __UTIL_CAPTURE_H__
#define __UTIL_CAPTURE_H__
#include <vector>
#include <list>
#include <map>
#include "service/pxcsmartasyncimpl.h"
#include "pxccapture.h"
#include "pxcsession.h"
#include "pxcsmartptr.h"

class UtilCapture {
public:

    friend class PXCSmartAsyncImplIxOy<UtilCapture,PXCImage,PXCCapture::VideoStream::STREAM_LIMIT,PXCScheduler::SyncPoint,1>;

    UtilCapture(PXCSession *session);
    virtual ~UtilCapture(void);

    virtual void SetFilter(PXCSession::ImplDesc &desc)                  { m_desc_filter=desc; }
    virtual void SetFilter(PXCImage::ImageType type, PXCSizeU32 &size)  { m_size_filters[type].push_back(size); }
    virtual void SetFilter(PXCCapture *capture)                         { m_capture_filter = capture; }
	virtual void SetFilter(const pxcCHAR *device)						{ m_name_filter=device; }

    virtual pxcStatus LocateStreams(PXCCapture::VideoStream::DataDesc *inputs);
    virtual pxcStatus LocateStreams(PXCCapture::AudioStream::DataDesc *inputs);
    virtual pxcStatus LocateStreams(std::vector<PXCCapture::VideoStream::DataDesc*> &minputs);
    virtual pxcStatus MapImages(int input, PXCImage *images_in[], PXCImage *images_out[]);
    virtual PXCImage  *QueryImage(PXCImage *images[], PXCImage::ImageType type);

    virtual PXCCapture::Device      *QueryDevice(void)              { return m_device; }
    virtual PXCCapture::VideoStream *QueryVideoStream(int channel, int input=0);
    virtual PXCCapture::AudioStream *QueryAudioStream(void)         { return m_astream; }
    virtual PXCCapture              *QueryCapture(void)             { return m_capture; }

    virtual pxcStatus ReadStreamAsync(PXCImage *images[], PXCScheduler::SyncPoint **sp);
    virtual pxcStatus ReadStreamAsync(PXCAudio **audio, PXCScheduler::SyncPoint **sp);

	virtual void Release(void) { delete this; }

protected:
    PXCSession*                 m_session;
    PXCSmartPtr<PXCScheduler>   m_scheduler;
    std::map<PXCImage::ImageType, std::list<PXCSizeU32> > m_size_filters;
    PXCSession::ImplDesc        m_desc_filter;
    const pxcCHAR*              m_name_filter;
    PXCCapture*                 m_capture_filter;

    PXCSmartPtr<PXCCapture>                                 m_capture;
    PXCSmartPtr<PXCCapture::Device>                         m_device;
    std::vector<PXCCapture::VideoStream*>                   m_vstreams;
    PXCSmartPtr<PXCCapture::AudioStream>                    m_astream;
    std::vector<PXCCapture::VideoStream::DataDesc*>         m_inputs;
    std::vector<std::vector<int>>                           m_maps;
   
    virtual pxcStatus PXCAPI CreateCapture(pxcU32 index, PXCCapture **capture);
    virtual pxcStatus PXCAPI ReadStreamSync(PXCImage *images[], PXCScheduler::SyncPoint *spts[]);
	virtual pxcStatus PXCAPI PassOnStatus(pxcStatus sts) { return sts; }
    int  MatchProfiles(PXCCapture::Device::StreamInfo &sinfo, std::list<PXCCapture::VideoStream::ProfileInfo> &profiles, int vstream);
    void ScanProfiles(std::list<PXCCapture::VideoStream::ProfileInfo> &profiles, PXCImage::ImageType stype, PXCCapture::VideoStream *vstream);
    bool RecordProperties(PXCCapture::Device *device);
    void FindBestProfile(std::list<PXCCapture::VideoStream::ProfileInfo> &profiles, int vstream);
    void DeleteVStreams(void);
    int  CalculateNumFormats(void);
    void ClearMaps(void);
};

#endif
